<?php

function XML_Template_process_file($filename, $obj, $clean = 1)
{
	$doc = new DOMDocument;
	$doc->load($filename);
	XML_Template_process($doc, $obj, $clean);
	return $doc;
}

function XML_Template_process($node, $obj, $clean = 1)
{
	if (is_a($obj, 'domnode')) {                          # overwrite
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$node->removeChild($child);
		}

		if (is_a($obj, 'domdocument')) {
			$obj = $obj->document_element();
		}

		$newobj = own_clone_node($obj, $node->ownerDocument);
		$node->appendChild($newobj);

		XML_Template_process($newobj, array(), $clean);
	} else if (!is_array($obj)) {                         # overwrite
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$node->removeChild($child);
		}
		$doc = $node->ownerDocument;
		$node->appendChild($doc->createTextNode($obj));
	} else if (is_associative_array($obj)) {              # substitute
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$processed = false;

			if ($child->nodeType == XML_ELEMENT_NODE) {
				unset($id);

				$tag = $child->nodeName;
				$attrs = $child->attributes;
				if (isset($attrs)) {
					foreach ($child->attributes as $attr) {
						if ($attr->namespaceURI == 'http://template.sesse.net/' && $attr->name == 'id') {
							$id = $attr->value;
							if ($clean) {
								$child->removeAttributeNode($attr);
							}
						}
					}
				}
			
				# check all substitutions to see if we found anything
				# appropriate
				foreach (array_keys($obj) as $key) {
					# FIXME: we would want something like \Q and \E here...
					if (preg_match('/^' . $tag . '\/(.*)$/', $key, $matches) ||
					    (isset($id) && preg_match('/^#' . $id . '\/(.*)$/', $key, $matches))) {
					    	$child->set_attribute($matches[1], $obj[$key]);
					}

					if ($processed) {
						continue;
					}
					if ($key == $tag || (isset($id) && $key == ('#'.$id))) {
						XML_Template_process($child, $obj[$key], $clean);
						$processed = true;
					}
				}
			}

			if (!$processed) {
				XML_Template_process($child, $obj, $clean);
			}
		}
	} else {						# repeat
		$doc = $node->ownerDocument;
		$frag = $doc->createElement("temporary-fragment");    # ugh

		while ($node->childNodes->length > 0) {
			$child = $node->childNodes->item(0);
			$frag->appendChild($child);
		}

		foreach ($obj as $instance) {
			if (!isset($instance)) {
				continue;
			}

			$newnode = own_clone_node($frag, $frag->ownerDocument);
			$node->appendChild($newnode);
			XML_Template_process($newnode, $instance, $clean);
			if ($clean) {
				XML_Template_clean($newnode);
			}
		}

		# remove all the <fragment> tags

		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			if ($child->name != 'temporary-fragment') {
				continue;
			}
			for ($j = 0; $j < $child->childNodes->length; ++$j) {
				$child2 = $child->childNodes->item($j);
				$node->appendChild($child2);
			}
			$node->removeChild($child);
		}	
	}

	if ($clean) {
		XML_Template_clean($node);
	}
}

function XML_Template_clean($node)
{
	if ($node->nodeType == XML_ELEMENT_NODE) {
		if ($node->namespaceURI != 'http://template.sesse.net/') {
			return;
		}

		# as this is a dummy node, we want to remove it and move everything further up
		# after we've done any required replacements
		$doc = $node->ownerDocument;
		$parent = $node->parentNode;
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$node->removeChild($child);
			$node->insert_before($child, $node);
		}
		$parent->removeChild($node);
	}
}

# FIXME: use varargs here
function XML_Template_alternate($tag, $array, $elems)
{
	$num = count($elems);

	for ($i = 0, $j = 0; $i < count($array); $i++) {
		if (isset($array[$i])) {
			$array[$i][$tag] = $elems[$j++ % $num];
		}
	}

	return $array;
}
		
# Ideally, this would be "return $obj->clone_node(true)". But surprise, surprise,
# PHP is buggy (at least both PHP4 and PHP5); it removes the prefix information
# from all attributes during a clone. IOW, we'll have to clone evverything
# ourselves.
function own_clone_node($node, $doc)
{
	// we only need these two
	if ($node->nodeType == XML_ELEMENT_NODE) {
		$nsuri = $node->namespaceURI;
		if (isset($nsuri)) {
			$newnode = $doc->createElementNS($node->namespaceURI, $node->nodeName, $node->prefix);
		} else {
			$newnode = $doc->createElement($node->nodeName);
		}
		
		$attrs = $node->attributes;
		if (isset($attrs)) {
			foreach ($node->attributes as $attr) {
				$nsuri = $attr->namespaceURI;
				if (isset($nsuri)) {
					$attr2 = $doc->createAttribute($attr->name);
				} else {
					$attr2 = $doc->createAttributeNS($nsuri, $attr->prefix . ":" . $attr->name);
				}
				$attr2->value = $attr->value;
				$newnode->appendChild($attr2);
			}
		}
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$newnode->appendChild(own_clone_node($child, $doc));
		}
		return $newnode;
	} else {
		return $node->cloneNode(true);
	}
}

function is_associative_array($arr)
{
	if (!is_array($arr)) {
		return false;
	}
	$diff = array_diff(range(0, count($arr) - 1), array_keys($arr));
	return (count($diff) > 0);
}
?>
